from typing import Callable

from .route import auto_generate_route


class AppDefinition:
    def __init__(
        self,
        name: str,
        handler: Callable,
        *,
        route: str | None = None,
        description: str | None = None,
        hidden: bool = False
    ):
        if not name:
            raise ValueError("Missing 'name' field in Compose.App constructor")

        if not handler:
            raise ValueError("Missing 'handler' field in Compose.App constructor")

        self.name = name
        self.handler = handler
        self.description = description
        self.hidden = hidden

        self._isAutoGeneratedRoute = route == None

        self._route = auto_generate_route(name) if self._isAutoGeneratedRoute else route

    @property
    def route(self) -> str:
        return self._route

    @property
    def is_auto_generated_route(self) -> bool:
        return self._isAutoGeneratedRoute

    def set_route(self, route: str, isAutoGenerated: bool) -> None:
        self._route = route
        self._isAutoGeneratedRoute = isAutoGenerated

    def summarize(self) -> str:
        return {
            "name": self.name,
            "route": self.route,
            "description": self.description,
            "hidden": self.hidden,
        }
